function [success, details] = Move_var( var, s )

% [success, details] = Move_var( var, settings )
%
% IVGA primitive operation. Recorded properties:
%   cputimer, cputime, total cost
%
% var   variable to be moved. if empty, a random variable will be chosen.
%
% settings, valid fields:
%   .reinit_lim_trigger    new groups' reiniting is stopped if either the
%                          estimated cost is reached or this trigger activates
%   .improve_lim_trigger   new groups' tuning is stopped if either the
%                          estimated cost is reached or this trigger activates
%
% Returns:
%   success   true if a good move was found and done, false otherwise.
%   details   the 'move' struct, which is defined briefly at the beginning
%             of the implementation.
%
% Tries to move the specified variable from its current group to all other
% groups, one by one. Creating a new group for the variable is also
% considered. The best alternative will be committed if it improves the
% total cost. If a move is committed, the new groups are tuned (with reinit
% and improve, in this order) until their cost reaches the estimate, or the
% corresponding trigger activates.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.



% 'old' groups represent the situation before the move operation
% and 'new' groups represent the situation after the operation:
%   old source group = source group which still contains var
%                      (old_source_ind = its cache index)
%   old target group = target group which doesn't yet contain var
%                      (old_target_ind = its cache index, might be [])
%   new source group = source group which doesn't anymore contain var
%                      (new_source_ind = its cache index, might be [])
%   new target group = target group which already contains var
%                      (new_target_ind = its cache index)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global STATS State;


STATS = measure( STATS, 'Move_var', 'start', State );
start_time = cputimer(STATS);

% retrieve the state transition limiter from the State
stl = State.state_transition_limiter;


% define the 'move' struct
move.e_min           = [];   % estimated cost delta
move.old_source_ind  = [];
move.old_target_ind  = [];   % might stay []
move.new_source_ind  = [];   % might stay []
move.new_target_ind  = [];
move.new_source_cost = [];   % estimated cost
move.new_target_cost = [];   % estimated cost
move.var             = [];   % index of the variable to be moved

% if no var given, choose one by random
if isempty( var )
  i = fix((rand-eps) * length(State.vars)) + 1;
  var = State.vars(i);
end

% store the variable index into the move struct
move.var = var;


% find the best move
move = find_best_move( var, move );

% was a valid move found?
if isfinite( move.e_min )
  
  % is the plan accepted?
  [stl, accept] = accept_move_plan( stl, State, 'Move_var', move );
  if accept
    
    % is the (estimated) result accepted? (note that the time delta is the
    % actual current delta and not an estimate for the whole process)
    [stl, accept] = accept_move_result( ...
        stl, State, cputimer(STATS) - start_time, move.e_min );
    if accept
      
      % commit the move
      move = commit_move( var, move );
      
      % try to improve new source and target groups up to the estimated costs
      improve_new_groups( move.new_source_ind, move.new_source_cost, ...
                          s.reinit_lim_trigger, s.improve_lim_trigger );
      improve_new_groups( move.new_target_ind, move.new_target_cost, ...
                          s.reinit_lim_trigger, s.improve_lim_trigger );
      
      % verify result
      move = verify_move( move );
      
      % is the true result accepted? (note that the time delta here is
      % different than the one passed on the previous accept_move_result call)
      [stl, accept] = accept_move_result( ...
          stl, State, cputimer(STATS) - start_time, move.e_min, ...
          'reuse_last_randomization' );
      if ~accept
        
        % revert if not accepted
        revert_move( move );
        
      end
      
    end
    
  end
  
end


% write the return values.
success = isfinite( move.e_min ) && accept;
details = move;

% store the state transition limiter back to the State
State.state_transition_limiter = stl;


STATS = measure( STATS, 'Move_var', 'checkpoint', State );
STATS = measure( STATS, 'Move_var', 'stop', State );
return;








function move = find_best_move( var, move )

% Finds the best way to move 'var' from its current group.
%
% If no move improves the cost, e_min will be zero.
%
% If the variable should be moved into an existing group, then
% 'old_target_ind' will be its index and 'new_target_ind' is empty.
%
% If the variable should be moved to a group of its own, then
% 'old_target_ind' will be empty and 'new_target_ind' will be the index of
% the new group, which will be also created and precomputed (but not
% activated). If the new group must be created, its model will be very low
% quality!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;

move.e_min = Inf;


% find var's current group
move.old_source_ind = State.Ind( find_group(State, var) );

% compute some info from the source group
src_group = State.Groups( move.old_source_ind );
src_cost  = get_cost( src_group.Model );
if length( src_group.Vars ) == 1
  move.new_source_cost = 0;
else
  move.new_source_cost = estimate_remove_vars( src_group.Model, var );
end


% consider existing groups
% (loop through all active groups except the var's current group)
inds = State.Ind; inds( find(inds==move.old_source_ind) ) = [];
for ind=inds
  
  % estimate the cost if the variable were moved into this group (=ind)
  this_old_cost = get_cost( State.Groups(ind).Model );
  this_new_cost = estimate_add_vars( State.Groups(ind).Model, ...
                                     src_group.Model, ...
                                     var );
  e = (move.new_source_cost + this_new_cost) - (src_cost + this_old_cost);
  
  % was this the best move so far?
  if e < move.e_min
    
    % store some info about it
    move.e_min           = e;
    move.old_target_ind  = ind;
    move.new_target_ind  = [];
    move.new_target_cost = this_new_cost;
    
  end
  
end

% consider creating a new group for the variable
% (Note: if the group must be created, its model will be very low quality!)
if length( src_group.Vars ) > 1
  
  % fetch/create the new target group and make sure it is in precomputed state
  ind = find_or_create( var );
  
  % estimate the resulting cost
  this_old_cost = 0;
  this_new_cost = get_cost( State.Groups(ind).Model );
  e = (move.new_source_cost + this_new_cost) - (src_cost + this_old_cost);
  
  % was this the best move so far?
  if e < move.e_min
    
    % store some info about it and leave the model in precomputed state
    move.e_min           = e;
    move.old_target_ind  = [];
    move.new_target_ind  = ind;
    move.new_target_cost = this_new_cost;
    
  else
    
    % make sure the model is purged
    State.Groups(ind).Model = purge( State.Groups(ind).Model );
    
  end
  
end

return;








function move = commit_move( var, move )

% Commits the specified variable move.
%
% Deactivate old source and target groups.
% Find/create new source and target groups.
% Activate new source and target groups.
%
% If the move leaves the var's current group empty, there is no
% 'new source group'.
% If the var is moved to a new group, there will be no 'old target group'.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;


% deactivate old source and target groups
set_group_active( move.old_source_ind, 0 );
if ~isempty( move.old_target_ind )
  set_group_active( move.old_target_ind, 0 );
end

% find/create new source group and activate it (if it will not be empty)
new_source_vars = State.Groups(move.old_source_ind).Vars;
new_source_vars( find(new_source_vars==var) ) = [];
if ~isempty( new_source_vars )
  move.new_source_ind = find_or_create(new_source_vars);
  set_group_active( move.new_source_ind, 1 );
end

% find/create new target group and activate it
if ~isempty( move.old_target_ind )
  new_target_vars = sort([ State.Groups(move.old_target_ind).Vars, var ]);
else
  new_target_vars = var;
end
move.new_target_ind = find_or_create( new_target_vars );
set_group_active( move.new_target_ind, 1 );

return;








function improve_new_groups( ind, target_cost, ...
                             reinit_lim_trigger, improve_lim_trigger )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;

if isempty(ind), return; end


% create stop triggers
cost_delta_needed = target_cost - get_cost( State.Groups(ind).Model );
ok_cost_range     = [-inf, total_cost(State) + cost_delta_needed];
reinit_trigger = trigger_or( ...
    trigger_range( ...
        ok_cost_range, ...
        flt_stats_reader( {'Move_var - Reinit_model', 'total cost'}, ...
                          'iters' ) ), ...
    reinit_lim_trigger );
improve_trigger = trigger_or( ...
    trigger_range( ...
        ok_cost_range, ...
        flt_stats_reader( {'Move_var - Improve_model', 'total cost'}, ...
                          'iters' ) ), ...
    improve_lim_trigger );

% reinit and improve
Reinit_model( ind, struct( 'stop_trigger', reinit_trigger ), 'Move_var' );
Improve_model( ind, struct( 'stop_trigger', improve_trigger ), 'Move_var' );

return;




function move = verify_move( move )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;

% real new costs
if ~isempty( move.new_source_ind )
  move.new_source_cost = get_cost( State.Groups(move.new_source_ind).Model );
  else move.new_source_cost = 0;
end
move.new_target_cost = get_cost( State.Groups(move.new_target_ind).Model );

% old costs
old_source_cost = get_cost( State.Groups(move.old_source_ind).Model );
if ~isempty( move.old_target_ind )
  old_target_cost = get_cost( State.Groups(move.old_target_ind).Model );
  else old_target_cost = 0;
end

% real e_min
move.e_min = (move.new_source_cost + move.new_target_cost) - ...
    (old_source_cost + old_target_cost);

return;




function revert_move( move )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;

% deactivate new group(s) and reactive old group(s)
if ~isempty(move.new_source_ind), set_group_active(move.new_source_ind, 0); end
set_group_active( move.new_target_ind, 0 );
set_group_active( move.old_source_ind, 1 );
if ~isempty(move.old_target_ind), set_group_active(move.old_target_ind, 1); end

return;
