function cs = cs_make_models( data, state, varargin )

% cs = cs_make_models( data, state, [options] )
%
% Computes models for the given grouping. No tuning of the grouping is
% performed.
%
% The quality of the models can be selected with the following option:
%   ['quality', q]
% where q is one of the strings 'best', 'good', 'normal', 'faster', 'fastest'.
%
% The runtime options specified in the help section of ivga() cannot be used
% with these settings.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% select the modeler and controller
cs.mdl.class  = 'mdl_vq_real_nominal';
cs.ctrl.class = 'ctrl_basic';




% ---- standard initialization ----


% parse the data dimensions and group setup
[n, dim, setup_mode, setup_params] = ...
    cshelper_parse_grouping_and_dim( cs, data, state, varargin{:} );

% store dimensionality for internal use
cs.private.n = n; cs.private.dim = dim;




% ---- make_models -specific initialization ----


% check and process the 'quality' level
[flg, quality] = isarg('quality'); if ~flg, quality = 'normal'; end
switch quality
 case 'best'
  reinits  = 50;
  improves = 10;
 case 'good'
  reinits  = 25;
  improves = 5;
 case 'normal'
  reinits  = 15;
  improves = 3;
 case 'faster'
  reinits  = 7;
  improves = 1;
 case 'fastest'
  reinits  = 1;
  improves = 0;
 otherwise
  if ~isscalar_(quality), error('Unknown quality level name!'); end
end




% ---- search strategies ----


% dummy stl
cs.stl = state_transition_limiter( ...
    'neighborhood_limiter', null_limiter, ...
    'gradient_limiter', null_limiter );




% ---- modeler ----


cs.mdl.max_points       = round(n/2);
cs.mdl.ncentroids_def   = 2;
cs.mdl.iters            = 30;
cs.mdl.check            = 10;




% ---- controller ----


% -- init_groups --

% group setup
cs.ctrl.init_groups.setup_mode    = setup_mode;
cs.ctrl.init_groups.setup_params  = setup_params;

% group model computation
cs.ctrl.init_groups.Reinit_model.stop_trigger    = ...
    trigger_iters( reinits, ...
                   flt_stats_reader('init_groups - Reinit_model','iters'));
cs.ctrl.init_groups.Improve_model.stop_trigger   = ...
    trigger_iters( improves, ...
                   flt_stats_reader('init_groups - Improve_model','iters'));


% disable tune_groups
cs.ctrl.tune_groups.stop_trigger = ...
    trigger_iters( 0, flt_stats_reader( 'tune_groups', 'iters' ) );




% ---- standard finalization ----


% apply common extra arguments
cs = cshelper_apply_common_exargs( cs, data, state, varargin{:} );


return;
